/* This file is part of Error Correctiong Codes Application.
 *
 * Ecc-Application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ecc-Application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecc-Application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/*
 *  ----------------------------------------------------------------------------
 *  Filename      : DndPanel.java
 *  ----------------------------------------------------------------------------
 *  Application   : [ k2j ]
 *  Author        : Thomas Suter (original version ch.ethz.dndpanel),
 *                  Raimond Reichert (this revised version)
 *  File created  : August 2001
 *  ----------------------------------------------------------------------------
 */


package ch.educeth.util.gui.dndpanel;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.event.MouseEvent;
import javax.swing.*;


/**
 *  DndPanel provides for drag and drop support of its child components - and
 *  only those. In contrast to Java's drag and drop implementation, DndPanel
 *  supports dragging only within itself. Draggable and/or dropable children
 *  must implement the LightwightDragDrop interface, and call the
 *  addThisMouseInputListener (child) method on DndPanel.
 *
 * @author     Raimond Reichert
 * @created    6. August 2001
 */
public class DndPanel extends JPanel {

  private final static BasicStroke DND_LINE_STROKE    = new BasicStroke (4, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
  private final static AlphaComposite DND_LINE_ALPHA  = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
  private final static AlphaComposite DND_IMAGE_ALPHA = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
  private final static Color DND_SUPPORTED     = new Color(0, 255, 0, 125);
  private final static Color DND_NOT_SUPPORTED = new Color(255, 0, 0, 125);
  private final static boolean ANTIALIASING    = true;
  private final static boolean PAINT_DRAG_IMAGE= true;
  private final static boolean PAINT_DRAG_LINE = true;

  /**
   * Where the current dragging started. pointSrc is relative to the origin
   * of this DndPanel.
   */
  private Point pointSrc;
  /**
   * The child component of DndPanel within which the dragging startet.
   */
  private Component dndSource;
  /**
   * X coordinate of the prigin of the dndSource child. Relative to this
   * DndPanel.
   */
  private int dndSourceX;
  /**
   * Y coordinate of the prigin of the dndSource child. Relative to this
   * DndPanel.
   */
  private int dndSourceY;
  /**
   * Where the current dragging endpoint is.
   */
  private Point pointDst;
  /**
   * Line object to connect pointSrc to pointDst.
   */
  private Line2D dndLine = new Line2D.Float();
  /**
   * Current color of dndLine. Set to DND_SUPPORTED or DND_NOT_SUPPORTED,
   * depending on whether the current pointDst is over a child which would
   * accept the dndObject.
   */
  private Color colorDND;
  /**
   * Reference to the data object which is to be dragged.
   */
  private Object dndObject;
  /**
   * Image representation of the dndObject.
   */
  private BufferedImage dndDragImage;
  /**
   * Width of dndDragImage.
   */
  private int dndDragImageWidth;
  /**
   * Height of dndDragImage.
   */
  private int dndDragImageHeight;

  private MouseMotionListener mouseMotionListener =
    new MouseMotionAdapter() {

      /**
       *  Description of the Method
       *
       * @param  e  Description of Parameter
       */
      public void mouseDragged(MouseEvent e) {
        if (dndSource != null) {
          pointDst.x = dndSourceX + e.getX();
          pointDst.y = dndSourceY + e.getY();

          colorDND = DND_NOT_SUPPORTED;

          Component child = findComponentAt(dndSourceX + e.getX(), dndSourceY + e.getY());

          if ((child != null) && (child instanceof LightweightDragDrop)) {
            int dndDestX = child.getLocationOnScreen().x - getLocationOnScreen().x;
            int dndDestY = child.getLocationOnScreen().y - getLocationOnScreen().y;

            MouseEvent event = new MouseEvent(
                dndSource, e.getID(), e.getWhen(), e.getModifiers(),
                dndSourceX + e.getX() - dndDestX, dndSourceY + e.getY() - dndDestY,
                e.getClickCount(), e.isPopupTrigger()
                );

            if (((LightweightDragDrop) child).acceptsDrop(event, dndObject)) {
              colorDND = DND_SUPPORTED;
            }
          }

          repaint();
        }
      }

    };

  private MouseListener mouseListener =
    new MouseAdapter() {

      /**
       *  Description of the Method
       *
       * @param  e  Description of Parameter
       */
      public void mousePressed(MouseEvent e) {
        if (e.getModifiers() == MouseEvent.BUTTON1_MASK) {
          Component child = (Component) e.getSource();

          if (child instanceof LightweightDragDrop) {
            dndSourceX = child.getLocationOnScreen().x - getLocationOnScreen().x;
            dndSourceY = child.getLocationOnScreen().y - getLocationOnScreen().y;

            LightweightDragDrop dndChild = (LightweightDragDrop) child;
            dndObject = dndChild.acceptsDrag(e);
            if (dndObject != null) {
              pointSrc = new Point(dndSourceX + e.getX(), dndSourceY + e.getY());
              pointDst = new Point(dndSourceX + e.getX(), dndSourceY + e.getY());

              dndSource = (Component) e.getSource();
              dndDragImage = dndChild.getDragImage(dndObject);
              dndDragImageWidth = dndDragImage.getWidth();
              dndDragImageHeight = dndDragImage.getHeight();
              colorDND = DND_SUPPORTED;
            }
          }
        }
      }


      /**
       *  Description of the Method
       *
       * @param  e  Description of Parameter
       */
      public void mouseReleased(MouseEvent e) {
        if (dndSource != null) {
          pointDst.x = dndSourceX + e.getX();
          pointDst.y = dndSourceY + e.getY();

          colorDND = DND_NOT_SUPPORTED;

          Component child = findComponentAt(dndSourceX + e.getX(), dndSourceY + e.getY());

          if ((child != null) && (child instanceof LightweightDragDrop)) {
            int dndDestX = child.getLocationOnScreen().x - getLocationOnScreen().x;
            int dndDestY = child.getLocationOnScreen().y - getLocationOnScreen().y;

            MouseEvent event = new MouseEvent(
                dndSource, e.getID(), e.getWhen(), e.getModifiers(),
                dndSourceX + e.getX() - dndDestX, dndSourceY + e.getY() - dndDestY,
                e.getClickCount(), e.isPopupTrigger()
                );

            if (((LightweightDragDrop) child).acceptsDrop(event, dndObject)) {
              ((LightweightDragDrop) child).dropTransferObject(event, dndObject);
              colorDND = DND_SUPPORTED;
            }
          }

          pointSrc = null;
          pointDst = null;
          dndSource = null;
          dndObject = null;
          dndDragImage = null;
          repaint();
        }
      }

    };

  /** @link aggregation */
  /*#LightweightDragDrop lnkLightweightDragDrop;*/

  /**
   */
  public DndPanel() {
    super();
  }


  /**
   *  <b>paint</b> paints the panel and all its children. Decides if the
   *  rubberband and the dragimage should be painted.
   *
   * @param  g  Graphics context for this component
   */
  public void paint(Graphics g) {
    super.paint(g);
    if (PAINT_DRAG_LINE) {
      paintDragLine((Graphics2D) g);
    }
    if (PAINT_DRAG_IMAGE) {
      paintDragImage((Graphics2D) g);
    }
  }


  /**
   *  <b>addThisMouseInputListener</b> adds this MouseInputListener as
   *  'MouseListener' and 'MouseInputListener' to the awt component 'c'.
   *
   * @param  component  The feature to be added to the ThisMouseInputListener
   *      attribute
   */
  public void addThisMouseInputListener(Component component) {
    component.addMouseListener(mouseListener);
    component.addMouseMotionListener(mouseMotionListener);
  }


  /**
   *  Description of the Method
   *
   * @param  component  Description of Parameter
   */
  public void removeThisMouseInputListener(Component component) {
    component.removeMouseListener(mouseListener);
    component.removeMouseMotionListener(mouseMotionListener);
  }


  /**
   *  <b>paintDragImage</b> paints the dragimage if it is available.
   *
   * @param  g  Description of Parameter
   */
  private void paintDragImage(Graphics2D g) {
    if (dndDragImage != null) {
      g.setComposite(DND_IMAGE_ALPHA);
      g.drawImage(dndDragImage, pointDst.x - dndDragImageWidth / 2, pointDst.y - dndDragImageHeight / 2, null);
      paintDragLine(g);
    }
  }


  /**
   *  <b>paintDragLine</b> paints the rubberband.
   *
   * @param  g  Description of Parameter
   */
  private void paintDragLine(Graphics2D g) {
    if (pointSrc != null && pointDst != null) {
      if (ANTIALIASING) {
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      }
      dndLine.setLine(pointSrc.getX(), pointSrc.getY(), pointDst.getX(), pointDst.getY());
      g.setPaintMode();
      g.setStroke(DND_LINE_STROKE);
      g.setPaint(colorDND);
      g.setComposite(DND_LINE_ALPHA);
      g.draw(dndLine);

      if (ANTIALIASING) {
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
      }
    }
  }

}
